@@ -147,11 +147,12 @@ test_that("filenames with shell characters can use relative resource paths", {
   on.exit(setwd(oldwd), add = TRUE)
 
   file.rename("resources/file-exists.Rmd", "resources/file exists.Rmd")
+  on.exit(file.rename("resources/file exists.Rmd", "resources/file-exists.Rmd"), add = TRUE)
+
   # render the file (contains an expression that stops if its resource is not
   # present)
   capture.output(output_file <- render("resources/file exists.Rmd"))
   on.exit(unlink(output_file), add = TRUE)
-  file.rename("resources/file exists.Rmd", "resources/file-exists.Rmd")
 })
 
 test_that("resources not deleted when filenames contain shell characters", {
@@ -196,3 +197,18 @@ test_that("resources are discovered in CSS files", {
   expected <- as.data.frame(expected[order(expected[[1]]), , drop = FALSE])
   expect_equal(resources, expected)
 })
+
+test_that("resources are discovered in notebook files", {
+  skip_on_cran()
+
+  resources <- find_external_resources("resources/r-notebook.Rmd")
+  expected <- data.frame(
+    path = c("tinyplot.png"),
+    explicit = c(FALSE),
+    web      = c(TRUE),
+    stringsAsFactors = FALSE)
+
+  resources <- as.data.frame(resources[order(resources[[1]]), , drop = FALSE])
+  expected <- as.data.frame(expected[order(expected[[1]]), , drop = FALSE])
+  expect_equal(resources, expected)
+})
